## ----setup, message=FALSE, warning=FALSE--------------------------------------
#| code-fold: true
#| code-summary: "Session Settings"
#| label: load-packages

# Graphs----
face_text='plain'
face_title='plain'
size_title = 14
size_text = 11
legend_size = 11

global_theme <- function() {
  theme_minimal() %+replace%
    theme(
      text = element_text(size = size_text, face = face_text),
      legend.position = "bottom",
      legend.direction = "horizontal", 
      legend.box = "vertical",
      legend.key = element_blank(),
      legend.text = element_text(size = legend_size),
      axis.text = element_text(size = size_text, face = face_text), 
      plot.title = element_text(
        size = size_title, 
        hjust = 0.5
      ),
      plot.subtitle = element_text(hjust = 0.5)
    )
}

# Outputs
options("digits" = 2)


## .justify {
##   text-align: justify !important
## }

## ----libraries, message=FALSE-------------------------------------------------

required_libraries <- c(
  "tidyverse", 
  "CASdatasets",
  "wesanderson",
  "mgcv",
  "broom",
  "knitr"
)
invisible(lapply(required_libraries, library, character.only = TRUE))


## ----load-gam-data, output=FALSE----------------------------------------------
#| code-fold: true
#| code-summary: "Code for importing our datasets"

data(beMTPL16)

CLAIMS <- beMTPL16

CLAIMS <- CLAIMS |>
  mutate(insured_age = 2016 - insured_birth_year)

CLAIMS <- CLAIMS |>
  group_by(insurance_contract, policy_year) |>
  mutate(ClaimNB = sum(number_of_liability_claims == 1))


## ----lambda_parameter, lambda_parameter---------------------------------------
#| code-fold: false

set.seed(1234) 

theoretic_count <- rpois(nrow(CLAIMS), mean(CLAIMS$ClaimNB))

tc_df <- tibble(theoretic_count)

freq_theoretic <- prop.table(table(tc_df$theoretic_count))

freq_claim <- prop.table(table(CLAIMS$ClaimNB))

freq_theoretic_df <- tibble(
  Count = as.numeric(names(freq_theoretic)),
  Frequency = as.numeric(freq_theoretic),
  Source = "Theoretical Count"
)

freq_claim_df <- tibble(
  Count = as.numeric(names(freq_claim)),
  Frequency = as.numeric(freq_claim),
  Source = "Empirical Count"
)

freq_combined <- freq_theoretic_df |> 
  rbind(freq_claim_df)


## ----poisson-claim------------------------------------------------------------
#| fig-cap: "<span style='color: #1E88E5;'>Theoretical</span> and <span style='color: black;'>empirical</span> histogram of claims in frequence"
#| label: "fig-plot-hist-claims"
#| code-fold: true
#| code-summary: "Code for the following graph"
#| fig-width: 8
#| fig-height: 5

ggplot(freq_combined, aes(x = Count, y = Frequency, fill = Source)) +
  geom_bar(stat = "identity", position = "dodge2", width = 0.3) +
  labs(x = "Claim Number", y = "Frequency", fill = "Legend") +
  theme(legend.position = "right") +
  scale_fill_manual(
    NULL,
    values = c("Empirical Count" = "black", "Theoretical Count" = "#1E88E5")
  ) +
  labs(fill = "Legend") +
  labs(x = "Claim Number", y = NULL) +
  theme(legend.position = "right")+
  global_theme()


## ----model--------------------------------------------------------------------
#| code-fold: false

reg <- gam(
  ClaimNB ~ -1 + s(insured_age) + s(vehicle_age) + offset(log(exposure)),
  family = quasipoisson,
  data = CLAIMS
)

summary(reg)


## ----Visualisation-perspective, message = FALSE-------------------------------
#| fig-cap: "3D impact perspective: DriverAge and vehicle_age"
#| label: "fig-plot-perspective"
#| code-fold: true
#| code-summary: Code to create the following graph
#| fig-width: 8
#| fig-height: 5

constant_offset <- mean(log(CLAIMS$exposure))


pgam <- function(x1, x2){
  predict(
    reg,
    newdata = data.frame(
      vehicle_age = x1, 
      insured_age = x2, 
      exposure = exp(constant_offset)
    ),
    type = "response"
  )
}

M <- 31
cx1 <- seq(from = min(CLAIMS$vehicle_age), to = max(CLAIMS$vehicle_age), length = M)
cx2 <- seq(from = min(CLAIMS$insured_age), to = max(CLAIMS$insured_age), length = M)


Pgam <- outer(cx1, cx2, Vectorize(pgam))


persp(
  cx1, cx2, Pgam, theta = 30, phi = 20,
  xlab = "Vehicle Age", ylab = "Insured Age", zlab = "Predicted Claims",
  main = paste(
    "Predicted Claims with Constant Offset (log(exposure) =", 
    round(constant_offset, 2), ")"
  )
)


## ----visualisation-coef-driverage---------------------------------------------
#| fig-cap: "Estimated effects of DriverAge"
#| label: "fig-plot-coef-driverage"
#| code-fold: true
#| code-summary: Code to create the following graph
#| fig-width: 8
#| fig-height: 5


plot(reg, select = 1)


## ----visualisation-coef-driverage---------------------------------------------
#| fig-cap: "Estimated effects of vehicle age"
#| label: "fig-plot-coef-vehicle-age"
#| code-fold: true
#| code-summary: Code to create the following graph
#| fig-width: 8
#| fig-height: 5

plot(reg, select = 2)

