## ----setup, message=FALSE, warning=FALSE--------------------------------------
#| code-fold: true
#| code-summary: "Session Settings"
#| label: load-packages

# Graphs----
face_text='plain'
face_title='plain'
size_title = 14
size_text = 11
legend_size = 11

global_theme <- function() {
  theme_minimal() %+replace%
    theme(
      text = element_text(size = size_text, face = face_text),
      legend.position = "bottom",
      legend.direction = "horizontal", 
      legend.box = "vertical",
      legend.key = element_blank(),
      legend.text = element_text(size = legend_size),
      axis.text = element_text(size = size_text, face = face_text), 
      plot.title = element_text(
        size = size_title, 
        hjust = 0.5
      ),
      plot.subtitle = element_text(hjust = 0.5)
    )
}

# Outputs
options("digits" = 2)


## .justify {
##   text-align: justify !important
## }

## ----libraries, message=FALSE-------------------------------------------------

required_libraries <- c(
  "CASdatasets",
  "tidyverse", 
  "demography",
  "rainbow",
  "forecast",
  "kableExtra",
  "RColorBrewer"
)
invisible(lapply(required_libraries, library, character.only = TRUE))


## ----import, echo = FALSE-----------------------------------------------------

data("freHMD")
france <- freHMD


## ----plot---------------------------------------------------------------------
#| code-fold: false
#| fig-cap: "Smoothed Male Log-Mortality Rates from 1900 to 2022 in France"
#| label: "fig-smoothed-male-mortality"
#| code-summary: "Code for plotting the male mortality"
#| fig-width: 8
#| fig-height: 5


france.sm <- smooth.demogdata(france, obs.var = "theoretical")

colors <- colorRampPalette(rainbow(length(france.sm$year)))(length(france.sm$year))

plot(france.sm, series = "male", type = "l",
     col = colors,
     main = "",
     ylab = "Log-Mortality Rate", xlab = "Age")


selected_years <- c(1900, 1916, 1943, 2000, 2020)
legend_colors <- colors[france.sm$year %in% selected_years]

legend("bottomright", legend = as.character(selected_years),
       col = legend_colors,
       lty = 1, 
       title = "Year", cex = 0.8)


## -----------------------------------------------------------------------------
#| code-fold: false

lc.male <- lca(france, series="male")
plot(lc.male)



## -----------------------------------------------------------------------------
#| code-fold: false

# Perform the forecast
forecast.lc.male <- forecast(lc.male, h = 20)


## -----------------------------------------------------------------------------
#| fig-cap: "Forecast Components for Male Mortality"
#| label: "fig-component-lee-male"
#| code-fold: true
#| code-summary: "Code for plotting the Lee-Carter components"
#| fig-width: 8
#| fig-height: 5

# Plot the forecast components
plot(forecast.lc.male, plot.type = "component", lwd = 2)



## -----------------------------------------------------------------------------
#| fig-cap: "Lee-Carter Method: Observed and Forecasted (next 20 years) Male Log-Mortality Rates in France"
#| label: "fig-forcast-lee-male"
#| code-fold: true
#| code-summary: "Code for plotting the Lee-Carter forecast"
#| fig-width: 8
#| fig-height: 5

# Plot the historical mortality data with adjusted line width and color
plot(france, series = "male", ylim = c(-13, 2), lty = 2, lwd = 1, col = "#440154FF",
     main = "",
     ylab = "Log-Mortality Rate", xlab = "Age")

# Add the forecast lines with distinct color and line type
lines(forecast.lc.male, col = "#1E88E5", lty = 1, lwd = 2)

# Add a legend to distinguish between observed and forecasted data
legend("bottomright", legend = c("Observed Data", "Forecast"),
       col = c("#440154FF", "#1E88E5"), lty = c(2, 1), lwd = 2, cex = 0.8)


## -----------------------------------------------------------------------------
#| code-fold: false

# filtering age <= 100 and years >= 1880
year_filter <- france$year >= 1880
filtered_years <- france$year[year_filter]
filtered_mortality_rates <- france$rate$male[, year_filter]

age_filter <- france$age <= 100
filtered_ages <- france$age[age_filter]
log_mortality_rates <- log(filtered_mortality_rates[age_filter, ])

# Create a Functional Data Set (fds) object for the fboxplot
log_mortality_fds <- fds(x = filtered_ages, y = log_mortality_rates)



## -----------------------------------------------------------------------------
#| fig-cap: "Functional Boxplot of Log-Mortality Rates in France"
#| label: "fig-fboxplot-functional-mortality"
#| code-fold: true
#| code-summary: "Code for the following functional HDR boxplot"
#| fig-width: 8
#| fig-height: 5
#| warning: false

fboxplot(data = log_mortality_fds,
         na.rm = TRUE,
         plot.type = "functional",   # For a functional boxplot
         type = "bag",               # For an bag boxplot
         alpha = c(0.05, 0.5),       # Coverage level
         projmethod = "PCAproj",     # Use PCA projection method
         factor = 1.96,              # Factor for the outer region
         xlab = "Age",               
         ylab = "Log-Mortality Rate",
         shadecols = gray((9:1)/10), # Colors for shaded regions
         pointcol = 1,               # Color of points (outliers and mode)
         plotlegend = TRUE,         
         legendpos = "bottomright",  
         ncol = 2)                  


## -----------------------------------------------------------------------------
#| fig-cap: "Bivariate Boxplot of Log-Mortality Rates in France"
#| label: "fig-fboxplot-bivariate-mortality"
#| code-fold: true
#| code-summary: "Code for the following functional HDR boxplot"
#| fig-width: 8
#| fig-height: 5

fboxplot(data = log_mortality_fds,
         na.rm = TRUE,
         plot.type = "bivariate",   # For a bivariate boxplot
         type = "bag",               # For an bag boxplot
         alpha = c(0.05, 0.5),       # Coverage level
         projmethod = "PCAproj",     # Use PCA projection method
         factor = 1.96,              # Factor for the outer region
         xlab = "Age",               
         ylab = "Log-Mortality Rate",
         shadecols = gray((9:1)/10), # Colors for shaded regions
         pointcol = 1,               # Color of points (outliers and mode)
         plotlegend = TRUE,         
         legendpos = "bottomright",  
         ncol = 2)                  


## -----------------------------------------------------------------------------
#| code-fold: false

# Apply the HUrob method using reflection-based PCA (RAPCA)
fdm_france <- fdm(france, series="male", method="rapca")

france.fcast <- forecast(fdm_france,20)



## -----------------------------------------------------------------------------
#| code-fold: true
#| fig-cap: "HUrob Method: Log-Mortality rates Observed and Forecasted (next 20 years) in France"
#| label: "fig-plot-forecast-mortality"
#| code-summary: "Code for the following forecast"
#| fig-width: 8
#| fig-height: 5

# Plot the observed and forecasted mortality rates
plot(france, series="male",
     main = "",
     ylim=c(-10, 0),
     col = "#440154FF",
     lty=2,
     ylab = "Log death rate", xlab = "Age")

lines(france.fcast, col = "#1E88E5", lty = 1, lwd = 2)

# Add a legend to distinguish between observed and forecasted data
legend("bottomright", legend = c("Observed Data", "Forecast"),
       col = c("#440154FF", "#1E88E5"), lty = c(2, 1), lwd = 2, cex = 0.8)


## -----------------------------------------------------------------------------
#| code-fold: false

models(france.fcast)

